#include "itext.h"
#include "xtext.h"
#include "xjp.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pwd.h>

#define FILEPATH_NO	10	


typedef struct CHUIM_DATA{

	char message[256];

	struct {

		char filepath[256];

#if HAVE_LIBIMLIB
		ImlibData *id;
		ImlibImage *image;

#endif

	}im[10];

	struct CHUIM_DATA *next;
	struct CHUIM_DATA *prev;

}chuim_data_t;

static chu_data_t *chead = NULL; 
static chu_data_t *ctail = NULL;

static chuim_data_t * _chuim_data_alloc()
{
	chuim_data_t *ct;
	int i;

	if((ct = (chuim_data_t *)malloc(sizeof(chuim_data_t))) == NULL){
	       	return NULL;
	}; 
	ct->message[0] = '\0';

	for(i=0; i < FILEPATH_NO; i++){
		ct->im[i].filepath[0] = '\0';

#if HAVE_LIBIMLIB
		ct->im[i].image = NULL;
#endif

	}

	return ct;
}
static chu_data_t * _chumessage_data_alloc()
{
	chu_data_t *ct;
	int i;

	if((ct = (chu_data_t *)malloc(sizeof(chu_data_t))) == NULL){
		fprintf(stderr,"Error: _chumeesage()(malloc() xtext_f5(14)\n");
	       	return NULL;
	}; 

	for(i=0; i < FILEPATH_NO; i++){
		ct->filepath[i][0] = '\0';
	}
	ct->message[0] = '\0';

	return ct;
}
static void _chuim_data_free(chuim_data_t **head,chuim_data_t **tail)
{
	chuim_data_t *ct,*ctt;

	if( *head == NULL) return;

	for(ct = *head; ct != NULL; ct = ctt){
		ctt = ct->next;

#if HAVE_LIBIMLIB
		int i;
		for(i = 0; i < FILEPATH_NO; i++){
			if( ct->im[i].image != NULL )
				Imlib_destroy_image(ct->im[i].id,ct->im[i].image);
		}
#endif
		free(ct);
	}
	*head = NULL;
	*tail = NULL;

	return;
}
static void _chumessage_data_free(chu_data_t **head,chu_data_t **tail)
{
	chu_data_t *ct,*ctt;

	for(ct = *head;ct != NULL;ct = ctt){ 
		ctt = ct->next; 

		free(ct); 
	}

	*head = NULL; *tail = NULL;

	return;
}
static void _chuim_data_add(chuim_data_t **head,chuim_data_t **tail,chuim_data_t *im)
{
        if(*tail){im->next = NULL; im->prev = *tail;(*tail)->next = im;*tail = im;}
        else{ im->next = NULL;im->prev = NULL;*tail = im;*head = im;}
}
static void _chumessage_data_add(chu_data_t **head,chu_data_t **tail,chu_data_t *im)
{
        if(*tail){im->next = NULL; im->prev = *tail;(*tail)->next = im;*tail = im;}
        else{ im->next = NULL;im->prev = NULL;*tail = im;*head = im;}
}

static int _check_sasie(chu_data_t *ct)
{
	struct stat sbuf;
	struct passwd *pwd;
	char dir[256];

	char *p,*p0,*p1,temp[128],temp1[128],temp2[128];
	int j = 0,len;

	if( ct == NULL ) return -1;

	pwd = getpwuid(getuid());
	sprintf(dir,"%s/%s",pwd->pw_dir,OTHER_FILE_DIR);

	p = ct->message;	

	for(;;){
		if( (p0 = strstr(p,"")) == NULL ){
		       	break;
		}
		if( (p1 = strstr(p0,"")) == NULL ){
			break;
		}else{
			memset(temp,'\0',127);
			strncpy(temp,p0+2,p1 - p0 -2);
			len = strlen(temp);

			Double2Single(temp1,temp,len);
			sprintf(temp2,"%s%s",dir,temp1);
			if( stat(temp2,&sbuf) == 0 )
				strcpy(ct->filepath[j++],temp2);
		}
		p = p1;
	}

	return 0;
}
static int xtext_draw_data(	Display *disp,
				Window cwin,
				my_window_t *win,
				chuim_data_t *ct,
				int ysize,
				int ypos	)
{
	int i,x,y;

	x = 5;
	y = ypos;

	for(i = 0; i < FILEPATH_NO; i++){

		if( ct->im[i].filepath[0] != '\0'){


			XmbDrawString(	disp,
					cwin,
					win->chu_fs,
					win->sgc,
					x,
					y,
					ct->im[i].filepath,
					strlen(ct->im[i].filepath)		
					);
			y += ysize;
		}

#if HAVE_LIBIMLIB
		if( ct->im[i].image != NULL ){
			int h;

			h = xtextImlibPasteImage(ct->im[i].id,ct->im[i].image,cwin,x,y);
			if( h < 0 ) continue;

			y += (h + ysize * 2);
		}
#endif

	}

	return y;
}
static int xtextDrawChuMessage(		Display *disp,
					Window cwin,
					my_window_t *win,
					chuim_data_t *head,
					chuim_data_t *tail,
					int step,
					unsigned int *mh		)
{
	chuim_data_t *ct;

	Window root;
	XRectangle ink,log;
	XFontSetExtents *fe;

	char *p,*pp,temp[256];
	int max,wx,wy;
	unsigned int bw,depth; 
	unsigned int w,h;
	int i,x,y,ypos; 
	int flag;
	int CSIZE,YSIZE;
	int step_per_pixel = 5;


	XClearWindow(disp,cwin);
	if(head == NULL) return step;

	strcpy(temp,head->message);

	if((fe = XExtentsOfFontSet(win->chu_fs)) == NULL){
		XmbTextExtents(win->chu_fs,temp,strlen(temp),&ink,&log);
	}else{
		ink = fe->max_ink_extent;
		log = fe->max_logical_extent;
	}

	CSIZE = log.width <= log.height ? log.width : log.height;
	YSIZE = CSIZE + 5;
	ypos = CSIZE + 2;

	XGetGeometry(disp,cwin,&root,&wx,&wy,&w,&h,&bw,&depth);

	x = 1; 
	y = step + ypos; 
	max = (w / CSIZE) * 2 - (CSIZE/2);

	for(ct=head;ct!=NULL;ct=ct->next){
		pp = ct->message;
		while(1){
			flag = 0; 

			for(p=pp,i=0;*p;p++,i++){

				if( *p & 0x80 ) flag = flag ? 0 : 1;

				if( i >= max ){
					if(flag == 0) temp[i++] = *(p++);
				       	break;
				}
				temp[i] = *p;
			}
			temp[i] = '\0'; 

			XmbTextExtents(win->chu_fs,temp,i,&ink,&log);
			if( log.width >= w ){
				fprintf(stderr,"Error: xtext_f5.c(100)\n");
				fprintf(stderr,"w:%d ink:%d log:%d\n",w,ink.width,log.width);
				fprintf(stderr,"%s\n",temp);
				break;
			}
			XmbDrawString(disp,cwin,win->chu_fs,win->fgc,x,y,temp,i);
			y += ypos;

			if(i<max)
				break;
			else
				pp = p;
		}

		y = xtext_draw_data(disp,cwin,win,ct,ypos,y);

	}
	*mh = y;

	if( y < h )
		step = step + step_per_pixel;
	if( step >= 0 )
		step = step_per_pixel * -1;


	return step;
}
void xtextClearChuMessage()
{
	if( chead != NULL )
		_chumessage_data_free(&chead,&ctail);

	return;
}
int xtextChuMessage(Display *disp,my_window_t *win,char *message)
{
	chu_data_t *ct;

	if( message == NULL) return -1;

	for( ct = chead; ct != NULL; ct = ct->next ){
	       if(strcmp(ct->message,message) == 0) return -1;
	}	       

	if((ct = (chu_data_t *)_chumessage_data_alloc()) == NULL){

		fprintf(stderr,"Error: chumessage xtext_f5.c(82)\n");
		return -1;

	}

	strcpy(ct->message,message);
	_check_sasie(ct);
	_chumessage_data_add(&chead,&ctail,ct);

	return 0;
}
void * xtextChuThreadWindow(void *my_win)
{
#if HAVE_LIBIMLIB

	ImlibData *id;
	ImlibImage *im;
#endif

	Display *disp;
	Window root,cwin;
	XSetWindowAttributes attr;
	XEvent ev;
	KeySym keysym;

	chuim_data_t *head = NULL,*tail = NULL,*ctt;
	chu_data_t *ct;

	my_window_t *mwin;

	unsigned int mh = 0;
	int width = 400,height = 200;
	int i,CHU_STEP = 0,loop = 1;
	char *title = "Messages";


	if( chead == NULL ) return NULL;

	disp = XOpenDisplay(NULL);
	root = DefaultRootWindow(disp);
	mwin = my_win;

#if HAVE_LIBIMLIB
	id = Imlib_init(disp);

	if( id->x.depth > 24 )
		id->x.depth = 24;
#endif

	for(ct = chead; ct != NULL; ct = ct->next){

		if((ctt = (chuim_data_t *)_chuim_data_alloc()) == NULL){
			fprintf(stderr,"Error: chu alloc xtext_f5(282)\n");
			continue;
		}

		strcpy(ctt->message,ct->message);

		for( i = 0; i < FILEPATH_NO; i++){
			if( ct->filepath[i][0] != '\0'){

				strcpy(ctt->im[i].filepath,ct->filepath[i]);

#if HAVE_LIBIMLIB
				ctt->im[i].id = id;
				if( (im = xtextImlibImage(disp,id,&(ctt->im[i].image),ct->filepath[i])) != NULL){
					if(im->rgb_width > width)
						width = im->rgb_width + 10;
				}
#endif
			}
		}

		_chuim_data_add(&head,&tail,ctt);

	}
		

	height = mwin->height;
	cwin = XCreateSimpleWindow(	disp,
					root,
					0,
					0,
					width,
					height,
					0,
					0,
					mwin->menu_bg_pixel	);


	XSelectInput(	disp,
			cwin,
			ExposureMask |
			StructureNotifyMask |
			KeyPressMask 
    					    );

	attr.backing_store = Always;
	XChangeWindowAttributes(disp,cwin,CWBackingStore,&attr);
	XDefineCursor(disp,cwin,mwin->pointer1);
	XStoreName(disp,cwin,title);

	xtextDrawChuMessage(disp,cwin,mwin,head,tail,CHU_STEP,&mh);
	if( mh < mwin->height )
		XResizeWindow(disp,cwin,width,mh);

	XMapWindow(disp,cwin);

	while(loop){
		XNextEvent(disp,&ev);
		switch(ev.type){
			case Expose:
				xtextDrawChuMessage(disp,cwin,mwin,head,tail,CHU_STEP,&mh);
			break;
			case KeyPress:
				keysym = XLookupKeysym(&ev.xkey,0);
				switch(keysym){
					case XK_Up:
						CHU_STEP += 5;
						CHU_STEP = xtextDrawChuMessage(disp,cwin,mwin,head,tail,CHU_STEP,&mh);
					break;
					case XK_Down:
						CHU_STEP -= 5;
						CHU_STEP = xtextDrawChuMessage(disp,cwin,mwin,head,tail,CHU_STEP,&mh);
					break;

					case XK_q:
						loop = 0;
					break;
				}
			break;
		}
	}
	_chuim_data_free(&head,&tail);

	XDestroyWindow(disp,cwin);

	/*
	XCloseDisplay(disp);
	*/

	XFlush(disp);

	return NULL;
}
